#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Calculate checksums with different algorithms in one pass.
"""
from __future__ import absolute_import, division, print_function
import sys
import hashlib
from argparse import ArgumentParser
from collections import namedtuple
from functools import partial

__author__ = "Marc 'BlackJack' Rintsch"
__version__ = '0.0.2'
__date__ = '2005-02-27'

__docformat__ = 'reStructuredText'

BLOCK_SIZE = 32 * 1024

Summer = namedtuple('Summer', 'path summer')


def calculate_checksums(algorithm_names, path, binary=False):
    """Calculate checksums with given algorithms for given paths.
    
    :param algorithm_names: Iterable of checksum algorithm names from the
        `hashlib` module.
    :param path: name of the file to calculate the checksum of.
    :type path: str
    :param binary: open the file in binary mode or not.
    :type binary: bool
    
    :raises IOError: if the file can't be opened or read.
    """
    path_prefix, mode = ('*', 'rb') if binary else (' ', 'r')
    summers = [
        Summer(path_prefix + path, hashlib.new(name))
        for name in algorithm_names
    ]
    infile = sys.stdin if path == '-' else open(path, mode)

    try:
        for block in iter(partial(infile.read, BLOCK_SIZE), ''):
            for summer in summers:
                summer.summer.update(block)
    finally:
        infile.close()
    
    return [(s.path, s.summer.hexdigest()) for s in summers]


def make_option_parser():
    """Create and populate an `ArgumentParser`."""
    parser = ArgumentParser(
        version=__version__,
        description='Calculate checksums.  Output is compatible with `md5sum`'
            'and `sha1sum`.',
        epilog='Copyright 2005, 2015 by {0}.'.format(__author__)
    )
    parser.add_argument(
        'filenames', metavar='FILE', nargs='*', default='-',
        help='File to calculate the checksum of.  `-` means stdin which is'
            ' also the default if no filenames are given.'
    )
    # 
    # TODO Add filename argument for '-' or no filename.
    # TODO Add algorithms argument.
    # TODO Add file mode argument.
    # 
    return parser


def main():
    """Main function."""
    options = make_option_parser().parse_args()
    
    for filename in options.filenames:
        try:
            for path, checksum in calculate_checksums(('md5', 'sha'), filename):
                print('{0} {1}'.format(checksum, path))
        except IOError, error:
            print(error, file=sys.stderr)


if __name__ == '__main__':
    main()
